#!/usr/bin/env python3
"""
Abiword Fuzzer using JRSFuzz
by Javantea
Nov 14, 2015
PDF Fuzzer using JRSFuzz
by Javantea
Oct 29, 2015
DNS Fuzzer using JRSFuzz
by Javantea
Oct 11, 2015
Based on SMTP Fuzzer using JRSFuzz
by Javantea
Oct 11, 2015
Based on SMTP Grammar Fuzzer
by Javantea
Sept 14, 2015

A very reasonable dumb fuzzer for PDF. It's a mutation fuzzer.
Provide the following values to fuzz a binary file you have captured:
python3 pdf_jrsfuzz1.py filename

TIME='%M kB max memory\nReal:%e seconds\n' time abiword -t 1page1.txt /home/jvoss/src/WebODF/programs/benchmark/1page.odt
43680 kB max memory
Real:0.24 seconds

TODO:
Unzip the file and modify each file separately.
CRC32 problems if I don't use zipfile
"""
import jrsfuzz
import sys
import debug1
import time
import tempfile
import argparse
import zipfile

def main():
	# This works on any document that abiword can parse
	parser = argparse.ArgumentParser(description='fuzz abiword')
	parser.add_argument('-v', '--valgrind', action='store_true')
	parser.add_argument('ddd', nargs="*")
	args = parser.parse_args(sys.argv[1:])
	data = b'%'
	ext = '.odt'
	valgrind = False
	if args.valgrind:
		valgrind = True
	#end if
	if len(args.ddd) > 0:
		filename = args.ddd[0]
		data = open(filename, 'rb').read()
		if '.' in filename:
			ext = filename[filename.rindex('.'):]
			if '/' in ext: raise Exception("Filename needs an extension.")
		#end if
	#end if
	lines_output = len(data) * 256
	print("%i outputs, %3.3f MB" % (lines_output, lines_output*len(data)/(1<<20)), file=sys.stderr)
	start, end = 0, lines_output
	if len(args.ddd) > 1:
		start = int(args.ddd[1])
	#end if
	if len(args.ddd) > 2:
		end = int(args.ddd[2])
	#end if
	if start > lines_output: start = lines_output-1
	if end > lines_output: end = lines_output
	text_outfile = tempfile.NamedTemporaryFile(suffix='.txt', prefix='fuzzfile', dir='.', delete=False)
	outfile = tempfile.NamedTemporaryFile(suffix=ext, prefix='fuzzfile', dir='.', delete=False)
	filename_out = outfile.name
	filename_text_out = text_outfile.name
	
	"""
	# TODO: figure this whole thing out.
	if ext == '.odt':
		odtFile = zipfile.ZipFile(filename)
		names = odtFile.NameToInfo.keys()
		if 'content.xml' in names:
			content = odtFile.read('content.xml')
		#end if
	#end if
	"""

	#prevResp = None
	for i in range(start, end):
		x = jrsfuzz.JRSFuzz(data, i)
		outfile.seek(0)
		outfile.write(x)
		outfile.flush()
		print(i)
		start = time.time()
		program = '/usr/bin/abiword'
		add_args = []
		if valgrind:
			program = '/usr/bin/valgrind'
			add_args = ['valgrind']
		#end if
		r = debug1.fork_exec(program, add_args + ['/usr/bin/abiword', '-t', filename_text_out, filename_out])
		end = time.time()
		if r == None:
			print("Failed to execute apparently.")
			continue
		#end if
		pid, sts = r
		print("Now I have control again.", (pid, sts))
		if sts == 11:
			print("Segmentation fault on iteration {}".format(i))
		#end if
		if (end-start) >= 3:
			print("Slow execution on iteration {}".format(i))
		#end if
	#next i
#end def main()

if __name__ == '__main__':
	main()
#end if
