#!/usr/bin/env python3
"""
GPG JRSFuzzer
by Javantea
Nov 12, 2015
Based on GPG Bit Flip Fuzzer
by Javantea
Nov 16, 2014

Based on Dumb PDF Fuzzer by Javantea April 6, 2010 13:30:15

Runs quickly, low coverage.

"""
import binascii
import subprocess
import time
import os
import jrsfuzz

def crc24_octets(octets):
	"""
	Implementation of CRC24 in Python by Javantea
	Based on RFC 2440 section 6.1.
	"""
	CRC24_INIT = 0xb704ce
	CRC24_POLY = 0x1864cfb
	crc = CRC24_INIT
	
	for c in octets:
		crc ^= c << 16;
		for i in range(8):
			crc <<= 1
			# Make sure it's still in 32-bit field.
			crc &= 0xffffffff
			if (crc & 0x1000000):
				crc ^= CRC24_POLY
			#end if
		#next i
	#next c
	return crc & 0xffffff
#end def crc24_octets(octets)

def bit_fuzz(buf, i):
	"""
	Flip the i_th bit on filename and save to filename.f#.pdf
	"""
	n = i % 8
	z = i // 8
	
	b = buf[z] ^ (1 << n)
	
	buf[z] = b
	return buf
#end def bit_fuzz(buf)

def main():
	import sys
	
	if not os.path.exists('.gpgfuzz'):
		os.mkdir('.gpgfuzz')
	#end if
	
	filename = 'msg.asc'
	if len(sys.argv) > 1:
		filename = sys.argv[1]
	#end if
	a = open(filename,'r').read().split('\n')
	header  = a.pop(0) + '\n'
	header += a.pop(0) + '\n'
	header += a.pop(0) + '\n'
	footer = a.pop()
	footer = a.pop() + '\n' + footer
	# Remove the crc24
	a.pop()
	b = ''.join(a)
	orig = list(binascii.a2b_base64(b))
	print("Fuzzing for %i rounds." % (len(orig)*256))
	for i in range(len(orig)*256):
		start = time.time()
		# Save a copy so we don't end up screwing up our original
		c = orig[::]
		f = bytes(jrsfuzz.JRSFuzz(c, i))
		body = binascii.b2a_base64(f)
		crc24_hex = '%06x' % crc24_octets(f)
		crc24_b64 = b'=' + binascii.b2a_base64(binascii.unhexlify(crc24_hex)) + b'\n'
		# Remove the --homedir=.gpgfuzz/ if you want to fuzz with your current user.
		# This improves the fuzzing by including all kinds of messyness including GUI.
		# 'gpg', '--homedir=.gpgfuzz/'
		p = subprocess.Popen(['gdb', '--batch', '-x', 'gdbinit.txt', 'gpg'], stdin=subprocess.PIPE)
		p.stdin.write(header.encode('ascii') + body + crc24_b64 + footer.encode('ascii'))
		p.stdin.close()
		p.wait()
		print(i, '%3.3fs' % (time.time()-start))
	#next i

	print("Finished.")
#end def main()

if __name__ == '__main__':
	main()
#end if
