#!/usr/bin/env python3
"""
JPEG2000 Fuzzer using JRSFuzz
by Javantea
Nov 14, 2015
Based on PDF Fuzzer using JRSFuzz
by Javantea
Oct 29, 2015
DNS Fuzzer using JRSFuzz
by Javantea
Oct 11, 2015
SMTP Fuzzer using JRSFuzz
by Javantea
Oct 11, 2015
SMTP Grammar Fuzzer
by Javantea
Sept 14, 2015

A very reasonable dumb fuzzer for PDF/JPEG2000. It's a mutation fuzzer.
Provide the following values to fuzz a binary file you have captured:
python3 jp2_jrsfuzz1.py filename

TIME='%M kB max memory\nReal:%e seconds\n' time pdfimages ~/recent/doorsigns.pdf doorsigns
45004 kB max memory
Real:2.07 seconds

TIME='%M kB max memory\nReal:%e seconds\n' time /usr/bin/j2k_to_image -i ~/recent/doorsigns-001.jp2 -o doorsigns-001b.ppm >/dev/null
13200 kB max memory
Real:0.13 seconds

TODO:
"""
from __future__ import print_function
import jrsfuzz
import sys
import debug1
import time
import tempfile

def main():
	# This works on any PDF, but should be used on files with JPEG2000 images.
	data = b'%'
	valgrind = False
	ext = '.jp2'
	if '-v' in sys.argv:
		valgrind = True
		sys.argv.remove('-v')
	#end if
	if len(sys.argv) > 1:
		filename = sys.argv[1]
		data = open(filename, 'rb').read()
		if '.' in filename:
			ext = filename[filename.rindex('.'):]
			if '/' in ext: raise Exception("Filename needs an extension.")
		#end if
	#end if
	lines_output = len(data) * 256
	print("%i outputs, %3.3f MB" % (lines_output, lines_output*len(data)/(1<<20)), file=sys.stderr)
	start, end = 0, lines_output
	if len(sys.argv) > 2:
		start = int(sys.argv[2])
	#end if
	if len(sys.argv) > 3:
		end = int(sys.argv[3])
	#end if
	if start > lines_output: start = lines_output-1
	if end > lines_output: end = lines_output
	outfile = tempfile.NamedTemporaryFile(suffix=ext, prefix='fuzzfile', dir='.', delete=False)
	filename_out = outfile.name
	print("To remove files generated by this fuzzer:")
	print("rm %s-*.ppm" % filename_out, file=sys.stderr)
	if ext == '.pdf':
		program = '/usr/bin/pdfimages'
	else:
		program = '/usr/bin/opj_decompress'
	#end if

	#prevResp = None
	for i in range(start, end):
		x = jrsfuzz.JRSFuzz(data, i)
		outfile.seek(0)
		outfile.write(x)
		outfile.flush()
		print(i)
		start = time.time()
		add_args = []
		if valgrind:
			program = '/usr/bin/valgrind'
			add_args = ['valgrind']
		#end if
		# TODO: Let the user choose. These two programs are different despite using the same library.
		# FIXME: Creates an extra junk output file.
		if ext == '.pdf':
			r = debug1.fork_exec(program, add_args + ['/usr/bin/pdfimages', filename_out, filename_out])
		else:
			# , devNull=True
			r = debug1.fork_exec(program, add_args + ['/usr/bin/opj_decompress', '-i', filename_out, '-o', filename_out + '.ppm'])
		#end if
		end = time.time()
		if r == None:
			print("Failed to execute apparently.")
			continue
		#end if
		pid, sts = r
		print("Now I have control again.", (pid, sts))
		if sts == 11:
			print("Segmentation fault on iteration {}".format(i))
		#end if
		if (end-start) >= 3:
			print("Slow execution on iteration {}".format(i))
		#end if
	#next i
#end def main()

if __name__ == '__main__':
	main()
#end if
