#!/usr/bin/env python3
"""
Javantea's Ridiculously Simple Fuzzer
by Javantea
Oct 11, 2015

jrsfuzz
"""
from __future__ import print_function
import sys

if sys.hexversion >= 0x3000000:
	toBytes = bytes
else:
	def toBytes(values):
		output = ''
		for c in values:
			if type(c) == str:
				output += c
			else:
				output += chr(c)
			#end if
		#next c
		return output
	#end def toBytes(values)
#end if

def JRSFuzz(data, i):
	"""
	JRSFuzz is a dumb fuzzer with a goal of fuzzing each byte as if it were a 
	type in a TLV. JRSFuzz fuzzes each byte separately leaving the other 
	bytes alone. One obvious benefit is the number of rounds is easy to 
	compute:
	256 * len(data)
	Therefore if you give it a number higher, it will raise an Exception.
	The values tested for b'a:' are:
	[b'\x00:', b'\x01:', b'\x02:', b'\x03:', ..., 
	 b'\xfd:', b'\xfe:', b'\xff:', 
	 b'a\x00', b'a\x01', b'a\x02', b'a\x03', ..., 
	 b'a\xfd', b'a\xfe', b'a\xff']
	As you can see with the example, this is an excellent short grammar 
	fuzzer, working better than most smart fuzzers. However, it lacks the 
	intelligence and finess of most smart fuzzers, not changing the length 
	of any value.
	Another benefit is the scaling. With data length 5, it has 1280 rounds. 
	With data length 5000, it has 1280000 rounds which is totally 
	reasonable for a dumb fuzzer.
	"""
	pos = i >> 8
	v = i & 0xff
	q = list(data)
	q[pos] = v
	return toBytes(q)
#end def JRSFuzz(data, i)

def main():
	import sys
	data = b'a:'
	if len(sys.argv) > 1:
		data = sys.argv[1].encode('utf-8')
	#end if
	for i in range(256 * len(data)):
		sys.stdout.buffer.write(JRSFuzz(data, i))
	#next i
#end def main()

if __name__ == '__main__':
    main()
#end if
