#!/usr/bin/env python3
"""
SMTP Fuzzer with STARTTLS using JRSFuzz
by Javantea
Oct 11, 2015
Based on SMTP Fuzzer using JRSFuzz
by Javantea
Oct 11, 2015
Based on SMTP Grammar Fuzzer
by Javantea
Sept 14, 2015

TODO:
DATA
Headers
Body
MIME
advanced SMTP commands.
Handle errors returned by the server.
Monitors with GDB and ASAN integration.
"""
import jrsfuzz
import sys
import socket
import ssl
import select

def main():
	# This works on a mail server that allows STARTTLS (suzy)
	# Data before the STARTTLS
	data = b'EHLO a.altsci.com\r\nSTARTTLS\r\n'
	# Data after the STARTTLS
	data2 = b'MAIL FROM: jvoss@altsci.com\r\nRCPT TO: jvoss@altsci.com\r\nQUIT\r\n'
	host, port = 'localhost', 25
	if len(sys.argv) > 1:
			host = sys.argv[1]
	#end if
	if len(sys.argv) > 2:
			port = int(sys.argv[2])
	#end if
	dest = (host, port)
	family, socktype = socket.AF_UNSPEC, socket.SOCK_STREAM
	gai = socket.getaddrinfo(host, port, family, socktype)
	
	if len(sys.argv) > 3:
		data2 = sys.argv[3]
	#end if
	if len(sys.argv) > 4:
		data = data2
		data2 = sys.argv[4]
	#end if
	lines_output = len(data + data2) * 256
	print("%i outputs, %3.3f MB" % (lines_output, lines_output*len(data + data2)/(1<<20)), file=sys.stderr)

	our_data = None
	for family, socktype, proto, canonname, addr in gai:
		try:
			s = socket.socket(family, socktype, proto)
			s.connect(addr)
		except Exception as e:
			print(e,)
			continue
		#end try
		our_data = (family, socktype, proto, canonname, addr)
		break
	#next family, socktype, proto, canonname, addr
	
	if our_data == None:
		print("Can't connect to dest", dest)
	#end if
	family, socktype, proto, canonname, addr = our_data
	
	# Now data is combined, we store the part we want to send before starttls.
	pre_len = len(data)
	data += data2
	
	header = s.recv(1024)
	print(header)
	prevResp = None
	for i in range(lines_output):
		x = jrsfuzz.JRSFuzz(data, i)
		# We might be sending 2 commands, but let it be.
		s = socket.socket(family, socktype, proto)
		s.connect(addr)
		header = s.recv(1024)
		print(header)
		try:
			s.send(x[:pre_len])
		except BrokenPipeError:
			print("Broken Pipe.", i, x)
		except ConnectionResetError:
			print("Conn Reset.", i, x)
		#end try
		try:
			resp = s.recv(1024)
		except ConnectionResetError:
			print("Conn Reset on recv.", i, x)
			resp = b''
		#end try
		# NOTE: Forget the security, we just want some TLS v1.2.
		try:
			ss = ssl.wrap_socket(s, ssl_version=ssl.PROTOCOL_TLSv1, ciphers='HIGH:!aNULL:!MD5:!CAMELLIA:!NULL:!DSS:!kDHr:!kDHd:!kDH:!3DES')
		except ssl.SSLEOFError as see:
			print('ssl.SSLEOFError:', see)
			print(i, x)
			continue
		except ssl.SSLError as se:
			print('ssl.SSLError:', se)
			print(i, x)
			continue
		#end try
		"""
		# One source of major slowdown.
		f = select.select([ss], [], [], 0.1)
		resp = b''
		if len(f) > 0 and len(f[0]) > 0:
			print("Never ever gonna get old.")
			try:
				resp = ss.recv(1024)
			except ConnectionResetError:
				print("Conn Reset on recv.", i, x)
				resp = b''
			#end try
		#end if
		print(resp)
		"""
		try:
			ss.send(x[pre_len:])
		except BrokenPipeError:
			print("Broken Pipe.", i, x)
		except ConnectionResetError:
			print("Conn Reset.", i, x)
		#end try
		# One source of major slowdown.
		f = select.select([ss], [], [], 0.01)
		resp = b''
		if len(f) > 0 and len(f[0]) > 0:
			try:
				resp = ss.recv(1024)
			except ConnectionResetError:
				print("Conn Reset on recv.", i, x)
				resp = b''
			#end try
		#end if
		print(resp)
		if resp != prevResp: print(i, x)
		prevResp = resp
		ss.close()
	#next i
#end def main()

if __name__ == '__main__':
	main()
#end if
